//******************************************************************************
//******************************************************************************
/**
* @file    IBPNGClient.h
* @author  Markus van Pinxteren
* @date    21.04.2010
*
* @brief   Interface class for the BPNGClient DLL
*/
// $Id$
//******************************************************************************
//******************************************************************************

#ifndef __IBPNGCLIENT_H__
#define __IBPNGCLIENT_H__

#include "BPNGDefines.h"
#include "IClientProperties.h"
#include "IBPNGClientListener.h"

extern "C" {
    //==================================================================================================
    /**
    \brief Interface class for the blue PiraT 2 client library

    IBPNGClient is the interface class of the blue PiraT Client library. To get access to a blue PiraT 2
    data logger you need a pointer to an inplementing instance of the IBPNGClient interface.
    Use getBPNGClient() to get such a pointer. This will create an implementing instance on the heap.
    To avoid conflict between different runtime libraries it is obligatory to release this object with
    its IBPNGClient::release() function when not needed any more. Don't call the delete operator directly
    on this pointer.
    */
    //==================================================================================================


    struct IBPNGClient
    {
        //==================================================================================================
        // Connection functions

        /// Scan network for logger
        /**
        This function sends one broadcast UDP messages via all network adapters
        and notifies the calling application about responding devices with the
        listener functions onBPNGDeviceDetected(), onBPNGDeviceDisappeared() and
        onBPNGDeviceStateChange() (see IBPNGClientListener.h). For each broadcast
        message sent, the function waits for 100ms for responding devices

        The first function call notifies about all found devices. All following
        calls on the same IBPNGClient instance will only notify about changes to
        the previous call.
        */
        virtual void WINAPI scanNetworkForLogger() = 0;

        /// Connect to logger with passed IP address
        /**
        \ingroup DownloadFunctions
        \ingroup ConversionFunctions

        While the logger is connected, it won't go to standy mode until the last
        IBPNGClient instance is disconnected.
        If connect fails the function will return 0. On success the return value is 1.
        In case of failure further information can be retrieved with getLastError().

        @param ipAddress IP address of the logger that should be connected
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI connectLogger(const char* ipAddress) = 0;

        /// Disconnect the currently connected logger
        virtual void WINAPI disconnectLogger() = 0;

        /// Check the logger connection, returns 1 for valid connection and 0 for no or broken connection
        virtual BOOL WINAPI isConnected() = 0;


        //==================================================================================================
        // Init functions

        /// Initialisation of download and online conversion process.
        /**
        For trace download and conversion directly from the device
        this function must be called after the logger is connected.

        Within this function the reference data base is downloaded and
        read. Please note that reading a large RDB may take some time, espacially
        in debug mode.

        Function will return 0 on failure and 1 on success.
        In case of failure further information can be retrieved with getLastError().

        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI initOnline() = 0;

        /// Initialisation of offline conversion process.
        /**
        For trace conversion from an offline data set
        this function must be called first.

        Within this function the reference data base is read. Please note that reading
        a large RDB may take some time, espacially in debug mode.

        Function will return 0 on failure and 1 on success.
        In case of failure further information can be retrieved with getLastError().

        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI initOffline(const char* path) = 0;

        //==================================================================================================
        // Download functions

        /// Download trace data
        /**
        Pass the size and the pointer to an array of DataSpan. Each span specifies either a time span or an
        index span from the reference data base's entry IDs (DataBaseEntryId).
        IBPNGClient::initOnline() must have been called before.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().

        If getLastError() returns BPNG_DOWNLOAD_ERRORS several errors occured.
        Use getNumDownloadErrors() and getDownloadError(int index) for detailed information.

        @param numSpans Size of the passed DataSpan array in second parameter
        @param dataSpans Array of DataSpan, specifying the time or ID spans that should be downloaded
        @param target Path to the target directory or ZIP file. A passed directory must be empty or not existing. A passed ZIP path must not exist.
        @param doSorting Specifies whether the traces from different logger-internal sources should be sorted to one output stream or not.
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI downloadDataSpans(uint16_t numSpans, DataSpan* dataSpans, const char* target, BOOL doSorting ) = 0;

        //==================================================================================================
        // Conversion functions

        /// Returns the pointer to a new conversion set. Deprecated, use static function createNewConversonSet instead
        virtual IConversionSet* WINAPI createNewConversionSet() = 0;

        /// Convert all data specified by conversionSet.
        /**
        Before data from a logger or an offline data set can be converted, IBPNGClient::initOnline() resp.
        IBPNGClient::initOffline() must have been called before.

        The data specified by conversionSet is converted to the passed target directory.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().

        If getLastError() returns BPNG_CONVERSION_ERRORS several errors occured.
        Use getNumConversionErrors() and getConversionError(int index) for detailed information.

        @param conversionSet conversion settings, see IConversionSet
        @param target target directory for the converted trace files.
        Depending on the passed ClientProperties the files may be stored in sub folders named by date
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI convertData(IConversionSet* conversionSet, const char* target) = 0;


        //==================================================================================================
        //  config functions

        /// Download the current logger configuration to the passed path
        /**
        If you download the current configuration from the data logger you get a zip Archive that
        contains all relevant XML and XSD files to modifiy the configuration in a valid way and
        reconfigurate the device with the reconfigLogger() function.

        Please note: It is up to you to ensure a valid configuration if you want to modify it with
        your own tools. You should only modify the xml and not the xsd files. "DeviceConfiguration.xml"
        and "FirmwareConfiguration.xml" should also not be modified. They specifiy all xml files that
        are mandatory to reconfigurate the data logger.
        You can validate the xml files with the supplied xsd files and a XML library of your
        choice. One possibility would be the XERCES library, see http://xerces.apache.org/xerces-c/

        @param path The path inclusive file name where to store the downloaded configuration ZIP file.
        */
        virtual BOOL WINAPI getConfig(const char* path) = 0;

        /// Reconfig logger with the zipped new configuration
        /**
        Reconfigurates the logger with the passed configuration. The ZIP archive can be either one that
        was downloaded with the getConfig() method, stored by the client software or a modified one.
        If you want to create your own configuration ZIP archive the structure of this file must be the
        same as of those mentioned above (xml files inside an "etc" directory). The abstract.txt file
        and all *.xsd files are optional.

        Please note: It is up to you to ensure a valid configuration if you want to modify it with
        your own tools. You should only modify the xml and not the xsd files. "DeviceConfiguration.xml"
        and "FirmwareConfiguration.xml" should also not be modified. They specifiy all xml files that
        are mandatory to reconfigurate the data logger.
        You can validate the xml files with the supplied xsd files and a XML library of your
        choice. One possibility would be the XERCES library, see http://xerces.apache.org/xerces-c/

        @param configZip Path to the zip file that contains the configuration.
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI reconfigLogger(const char* configZip) = 0;

        /// Reconfig logger with the default configuration
        /**
        An invalid configuration will set the logger in error state. To fix this one possibility
        is to set the logger's default configuration.

        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setDefaultConfig() = 0;


        //==================================================================================================
        //  miscellaneous functions

        /// Get list of all events from the RDB.
        /**
        If initOnline() was called before, the events of the logger's RDB is returned.
        If initOffline() was called before, the events of the RDB included in the offline data set is returned.
        @return Pointer to a IRdbEventList
        */
        virtual IRdbEventList* WINAPI getEventList() = 0;

        /// Return the instance name passed to the getBPNGClient() function
        virtual const char* WINAPI getInstanceName() = 0;

        /// Returns the instance ID that is unique for all IBPNGClient instances in one process
        virtual int WINAPI getInstanceId() = 0;

        /// Get path to the reference data base
        /**
        After calling one of the init functions IBPNGClient::initOnline() or IBPNGClient::initOffline()
        this function returns the path to the current Reference Data Base of the logger resp. the
        offline data set. For online processes, the RDB is downloaded from the logger to a tmp directory.
        For offline processes from a ZIP archive, the RDB is extracted to a tmp directory. For offline
        processes from a directory this function just returns the path to the RDB inside this directory.

        @return Path to the downloaded or extracted RDB file
        @sa initOnline(), initOffline()
        */
        virtual const char* WINAPI getReferenceDataBasePath() = 0;

        /// Get path to the config directory (after calling one of the init functions)
        virtual const char* WINAPI getConfigPath() = 0;

        /// Get name of device
        /**
        After calling one of the init functions IBPNGClient::initOnline() or IBPNGClient::initOffline()
        this function returns the currently configured device name.

        @return The device name
        @sa initOnline(), initOffline()
        */
        virtual const char* WINAPI getDeviceName() = 0;

        /// Returns pointer to a channel list interface.
        /**
        After calling one of the init functions IBPNGClient::initOnline() or IBPNGClient::initOffline()
        this function returns a pointer to the logger's resp. offline data set's channel list.

        In case of error null is returned and further information can be retrieved with getLastError().

        @sa IChannelList
        */
        virtual const IChannelList* WINAPI getLoggerChannels() = 0;

        /// Get the firmware and hardware version string
        virtual const char* WINAPI getVersions() = 0;

        /// Update firmware
        /**
        This function updates the logger's firmware. An internal version check is done.
        If the second parameter \a force is 0 only firmware components with an older
        version then the component's version inside the firmware packet will be updated.

        @param fwPath Path to the firmware packet file that should be installed.
        @param force Flag whether to update the components independently from the components' versions
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI updateFirmware(const char* fwPath, BOOL force) = 0;

        /// Update licenses
        /**
        Overwrites the current license file with the new one.

        @param licenseFilePath Path to the new license file
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI updateLicenses(const char* licenseFilePath)=0;

        /// Returns the license file's content as string
        virtual const char* WINAPI getLicenses() = 0;

        /// Removes the current license file from the logger.
        virtual BOOL WINAPI removeAllLicenses() = 0;


        /// Delete trace data
        /**
        Pass the size and the pointer to an array of DataSpan. Each span specifies either a time span or an
        index span from the reference data base's entry IDs (DataBaseEntryId). If you want to create spans with
        those IDs you have to call the initOnline() function first to get the current RDB file.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().

        @param numSpans Size of the passed DataSpan array in second parameter
        @param dataSpans Array of DataSpan, specifying the time or ID spans that should be downloaded
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI deleteData(uint16_t numSpans, DataSpan* dataSpans)  = 0;

        /// Delete all trace data on the logger
        /**
        In case of failure further information can be retrieved with getLastError().
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI deleteAllData() = 0;

        /// Set an info event with the passed string on the connected logger.
        /**
        You can set an info event to the RDB. This event will be from type INFO and
        the passed message is written to the event's comment column

        @return Returns 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setInfoEvent(const char* msg) = 0;

        /// Set a marker on the connected logger. Returns 0 on error.
        /**
        You can set an marker to the RDB. The set event will be from type MARKER.

        @return Returns 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setMarker() = 0;

        /// Returns the current loggertime in seconds since 01.01.1970 UTC
        virtual uint64_t WINAPI getCurrentLoggerTime() = 0;

        /// Set logger time and date to the passed UTC time stamp
        /**
        The parameter time must be in seconds since 01.01.1970 UTC
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setTime(uint64_t time) = 0;

        /// Call this to keep logger alive
        /**
        The blue PiraT 2 data logger can be configured to go to standby after a
        specified timeout without any bus traffic on the connected interfaces.
        If you want to have access to a device without bus traffic, and you don't
        want to connect to it with connectLogger() you have to
        keep it alive by calling this function. This will start a separate thread
        that sends periodically ping messages to the passed IP address. Receiving
        these ping messages, the firmware will not shutdown the system.

        @param ip The IP address of the logger that should be kept alive
        @sa stopKeepLoggerAlive()
        */
        virtual void WINAPI keepLoggerAlive(const char* ip) = 0;

        /// Called to stop sending keep alive pings to the logger specified via the passed \a ip
        virtual void WINAPI stopKeepLoggerAlive(const char* ip) = 0;


        /// Return pointer to a format list interface. Returns null in case of error
        /**
        All formats returned by this function are available for data conversion.
        @sa IFormatList, IFormatInfo
        */
        virtual IFormatList* WINAPI getAvailableFormats() = 0;

        /// Let the connected device blink its front LEDs for identification
        /**
        You can use this function to identify you device if you can't identify it
        over the Name or IP address given from the IBPNGClientListener::onBPNGDeviceDetected callback function.
        */
        virtual void WINAPI flashDeviceLED() = 0;

        /// Parse CCPXCPMeasurement.xml and CCPXCPConfiguration.xml and create CCPXCPSequence.xml
        virtual int WINAPI createCCPXCPSeqFile(const char* xsdDir, const char* xmlDir, bool forceFlag) = 0;

        /// Parse CCPXCPMeasurement.xml and CCPXCPConfiguration.xml and create a Vector DBC file for each device
        virtual int WINAPI createCCPXCPDbcFiles(const char* dbcDir, const char* xsdDir, const char* xmlDir) = 0;

        /// Return pointer to a false measure signal list interface.
        /**
        After calling the IBPNGClient::createCCPXCPDbcFiles() this function returns a pointer to a list with
        all measure signals which were ignored at DBC file generation.

        @sa IFalseMeasureSignal
        */
        virtual const IFalseMeasureSignalList* WINAPI getFalseMeasureSignals() = 0;

        /// Add a listener
        virtual void WINAPI addListener(IBPNGClientListener* listener) = 0;

        /// Remove a listener
        virtual void WINAPI removeListener(IBPNGClientListener* listener) = 0;

        /// Get last error code
        /**
        If any called BPNGClient function returns a value that indicates an error
        you can retrieve further information about that error with this function.
        @return The error description with error code and optional string value.
        @sa BPNGError
        */
        virtual BPNGError WINAPI getLastError() = 0;

        /// Returns the number of errors occured during the last conversion process.
        /**
        If convertData() fails, getLastError() can return different kinds of errors.
        There are types of errors that won't interrupt the conversion process but will be
        gathered during conversion and notified at the end. In that case the error code
        returned by getLastError() will be BPNG_CONVERSION_ERRORS and you can get the
        number of errors with this function.
        @sa getConversionError()
        */
        virtual int WINAPI getNumConversionErrors() = 0;

        /// Returns the conversion error at \a index.
        /**
        After getting the number of conversion errors with getNumConversionErrors()
        you can get all single errors with this function.
        */
        virtual BPNGError WINAPI getConversionError(int index) = 0;

        /// Returns the number of errors occured during the last download process.
        /**
        If downloadDataSpans() fails, getLastError() can return different kinds of errors.
        There are types of errors that won't interrupt the download process but will be
        gathered during download and notified at the end. In that case the error code
        returned by getLastError() will be BPNG_DOWNLOAD_ERRORS and you can get the
        number of errors with this function.
        @sa getDownloadError()
        */
        virtual int WINAPI getNumDownloadErrors() = 0;

        /// Returns the download error at \a index.
        /**
        After getting the number of download errors with getNumDownloadErrors()
        you can get all single errors with this function.
        */
        virtual BPNGError WINAPI getDownloadError(int index) = 0;

        /// Returns the current client library version
        virtual const char* WINAPI getLibVersion() = 0;

        /// Free memory of this IBPNGClient instance
        /**
        With the call of getBPNGClient() a new instance is created on the heap. The user is responsible to free its
        memory if it isn't needed any more. This function calls the delete operator on itself.

        Important note: Any further function call on the IBPNGClient instance after release() was called
        will cause a memory access violation and will crash the application!
        */
        virtual void WINAPI release() = 0;

        ///  Returns the current properties, deprecated use \a getClientProperties() instead
        /**
        @sa ClientProperties, setProperties()
        */
        virtual ClientProperties WINAPI getProperties() = 0;

        /// Set client properties, deprecated use \a setClientProperties(IClientProperties*) instead
        /**
        @sa ClientProperties, getProperties()
        */
        virtual void WINAPI setProperties(ClientProperties val) = 0;

        // Returns a copy of the current client properties
        /**
          @sa IClientProperties, setClientProperties()
        */
        virtual IClientProperties* WINAPI getClientProperties() = 0;

        // Set client properties
        /**
          @param Pointer to IClientProperties which can be retrieved from the static function \a createNewClientProperties()
                  or from \a IBPNGClient::getClientProperties()
          @sa IClientProperties, getClientProperties(), createNewClientProperties
        */
        virtual void WINAPI setClientProperties(IClientProperties* properties) = 0;

        /// Save properties to ini file
        virtual void WINAPI saveProperties(const char* pathToIniFile) = 0;

        /// Load properties from ini file
        virtual void WINAPI loadProperties(const char* pathToIniFile) = 0;

        /// Remove all DBC file assignments
        virtual void WINAPI clearDBCFileAssignments() = 0;

        /// Assign a DBC file to a CAN channel. Multiple files for one CAN channel are allowed, but double used message IDs will ignored.
        /**
        @param channelIndexCAN Zero based CAN channel index
        @param dbcFilePath Absolute path to the dbc file
        */
        virtual void WINAPI assignDBCFile(int channelIndexCAN, const char* dbcFilePath) = 0;

        /// Reset marker counter
        virtual int WINAPI resetMarkerCounter() = 0;

        /// Download bug report
        /**
        The downloaded bug report is a ZIP archive with several log data and system files for
        error analyzing purposes.

        @param targetPath Path inclusive file name under that the bug report will be stored.
        @param mode that specifies what kind of data should be included in the report, @sa BPNGBugreportMode
        @param startTime Start time for the time span of trace data that should be included (usec since 01.01.1970 UTC). Only for mode BR_FULL_ALL_TRACES and BR_FULL_TIMESPAN_TRACES
        @param endTime End time for the time span of trace data that should be included (usec since 01.01.1970 UTC). Only for mode BR_FULL_ALL_TRACES and BR_FULL_TIMESPAN_TRACES
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI downloadBugReport(const char* targetPath, BPNGBugreportMode mode, uint64_t startTime, uint64_t endTime) = 0;
    };

    /// Factory function that creates instances of BPNGClient givin away ownership.
    /**
    The instance is created on the heap and the allocated memory must be freed by the calling application.
    You can pass a name to this function. This will be the name of the created instance.
    @sa IBPNGClient::release(), IBPNGClient::getInstanceName()
    */
    DECLDIR IBPNGClient* WINAPI getBPNGClient(const char* name = "");

    /// Factory function that creates instances of BpngCloudClients
    /**
    The instance is created on the heap and the allocated memory must be freed by the calling application.
    You can pass a name to this function. This will be the name of the created instance.
    @sa IBPNGClient::release(), IBPNGClient::getInstanceName()
    */
    DECLDIR IBPNGClient* WINAPI getTSLClient(const char* name = "");

    /// Sets the language for status messages
    DECLDIR void WINAPI setLanguageID(LanguageID id);

    /// returns a new created conversionset
    DECLDIR IConversionSet* WINAPI createNewConversionSet();

    /// To free memory of a IConversionSet, use this method. Otherwise the memory will be freed when detaching the DLL from process.
    /// Never call any function of an IConversionSet after passing the pointer to the freeConversionSetMemory function.
    /// This would cause a heap corruptions.
    DECLDIR void WINAPI freeConversionSetMemory(IConversionSet* convSet);

    // Creates and returns a pointer to a new ClientProperties object with default settings
    /**
        After modifying the properties, you can set them to an instance of
        IBPNGClient with setClientProperties();
        @sa IClientProperties, setClientProperties()
    */
    DECLDIR IClientProperties* WINAPI createNewClientProperties();

    /// To free memory of IClientProperties, use this method. Otherwise the memory will be freed when detaching the DLL from process.
    /// Never call any function of an IClientProperties pointer after passing the pointer to the freeClientPropertiesMemory function.
    /// This would cause a heap corruptions.
    DECLDIR void WINAPI freeClientPropertiesMemory(IClientProperties* prop);

    /// Adds a log listener to the library
    /**
    The BPNG library writes debug traces to std::cout.
    If you want to process the debug outputs additionally (e.g. write to file)
    you can set a log listener to the lib. All set listeners get the log outputs
    from all BPNGClient instances.

    All log outputs are forwarded to the registered listeners by calling
    the onLogRequest function that was added.

    IMPORTANT NOTE: Don't write anything to cout or cerr in your implementation of onLogRequest!
    This would lead to a recursive call of the forwarding function.

    @sa onLogRequest
    */
    DECLDIR void WINAPI addLogListener(onLogRequest logFunc);

    /// Removes a log listener from the library
    DECLDIR void WINAPI removeLogListener(onLogRequest logFunc);

    /// Set whether to write debug output to std::cout
    DECLDIR void WINAPI writeLogToCout(bool flag);

} // end of extern_C

#endif // __IBPNGCLIENT_H__